//=======================================================================
// utility.h : The set of utilities used by main.cpp
//
// Copyright (c) 2009 Tai-Pang Wu (pang@cse.ust.hk) and HKUST x CUHK
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//======================================================================= 

#ifndef _UTILITY_H_
#define _UTILITY_H_

#include <list>
using namespace std;

// viewworld read/write (for point only)
// point            : 3D point read from or write to the viewworld file
// numPoint         : number of point
// filename         : path of input/output viewworld file
// cleanPoint       : the list of bool indicating whether the corresponding point should be output.
//                    "true = output" and "false = do not output"
int readVW3DPoint( double**& point, int& numPoint, const char* filename );
int saveVW3DPoint( double** point, int numPoint, const char* filename, bool* cleanPoint = NULL );

// viewworld write line segments
// endPt1           : array of end point 1
// endPt2           : array of end point 2
// numLineSeg       : number of line segment
// filename         : path of the output viewworld file
int saveVWLineSeg( double** endPt1, double** endPt2, int numLineSeg, const char* filename );

// Conver tensors to principal axes
// endPt1[output]   : array of end point 1
// endPt2[output]   : array of end point 2
// numLineSeg       : number of line segment
// K                : array of input tensors
// x                : array of input points
// numPoint         : number of input point
// dim              : dimension of point
// axisScale        : the scale of principal axes
int tensors2Axis( double**& endPt1, double**& endPt2, int& numLineSeg, double*** K, double** x, int numPoint, int dim, double axisScale = 1.0);

// read/write bundler. For detail, please refer to Snavely's Bundler. The naming should be corresponding.
int readBundle( const char* filename, double**& point, double**& pointRGB, 
			   list<int>*& visibleCam, list<int>*& siftKey, list<double>*& ix, list<double>*& iy, int& numPoint, 
			   double*& focalLength, double**& radial, double***& R, double**& t, int& numCamera);
int saveBundle( const char* filename, double** point, double** pointRGB, 
			   list<int>* visibleCam, list<int>* siftKey, list<double>* ix, list<double>* iy, int numPoint, 
			   double* focalLength, double** radial, double*** R, double** t, int numCamera,
			   bool* cleanPoint = NULL );

// Normalize tensors
// Operation: Find the maximum eigenvalue from the pool of tensors. All the
//            tensors will be normalized by this eigenvalue.
//
// K[input/output]  : Array of input tensors
// numPoint         : Number of input tensor
// dim              : Dimension of the input point
int normalizeTensors( double*** K, int numPoint, int dim );

// Filter out noise by thresholding
//
// clean[output]    : An array of bool indicating whether the corresponding tensors are outlier or not
//                    "true = inlier" and "false = outlier"
// K                : Array of input tensors
// numPoint         : Number of input tensor
// dim              : Dimension of the input point
// threshold        : Threshold in [0,1]
int filterNoise( bool*& clean, double*** K, int numPoint, int dim, double threshold );

#endif

