//=======================================================================
// StdAfx.cpp : The implementation of the common functions
//
// Copyright (c) 2009 Tai-Pang Wu (pang@cse.ust.hk) and HKUST x CUHK
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//=======================================================================

// stdafx.cpp : source file that includes just the standard includes
// data_generator.pch will be the pre-compiled header
// stdafx.obj will contain the pre-compiled type information

#include "StdAfx.h"

// TODO: reference any additional headers you need in STDAFX.H
// and not in this file

double** allocate2DDouble( int y, int x )
{
	double** mat = new double*[y];
	int i;
	for( i =0; i<y; i++)
	{
		mat[i] = new double[x];
		memset(mat[i], 0, sizeof(double) * x );
	}

	return mat;
}

void safeDelete2DDouble( int y, double**& mat )
{
	int i;
	if( mat != NULL )
	{
		for( i = 0; i<y; i++)
		{
			SAFEDELARR( mat[i] );
		}
		SAFEDELARR( mat );
	}
}

void copyMatrix( double** from, double** to, int nRows, int nCols )
{
	int i, j;
	for( i=0; i<nCols; i++ )
		for( j=0; j<nRows; j++)
		{
			to[j][i] = from[j][i];
//			printf("to[%d][%d] = %lf\n", j, i, to[j][i]);
		}
}

double matrixDet( double **a, int n )
{
	int i,j,j1,j2;
	double det = 0;
	double **m = NULL;

	if (n < 1) { /* Error */
		printf( "Det matrix size < 1!!!\n" );
	} 
	else if (n == 1) 
	{ 
		/* Shouldn't get used */
		det = a[0][0];
	} 
	else if (n == 2) 
	{
		det = a[0][0] * a[1][1] - a[1][0] * a[0][1];
	} 
	else
	{
		det = 0;
		for (j1=0;j1<n;j1++) 
		{
			m = allocate2DDouble( n-1, n-1 ); 
			for (i=1;i<n;i++) 
			{
				j2 = 0;
				for (j=0;j<n;j++) 
				{
					if (j == j1) continue;
					m[i-1][j2] = a[i][j];
					j2++;
				}
			}
			
			det += pow(-1.0,1.0+j1+1.0) * a[0][j1] * matrixDet(m,n-1);
			safeDelete2DDouble( n-1, m );
		}
   }
   return(det);
}

double pdMatMaxEigV( double** a, int n, int maxIter )
{
	int i = 0;
	double* u = new double[ n ];
	double* v = new double[ n ];
	double  prev_p = -MYINFINITE;
	double  p = 0;

	// initialize u
	memset( u, 0, sizeof( double ) * n );
	u[0] = 1.0;

	bool finished = false;
	int iter = 0;
	while ( !finished && (iter < maxIter) )
	{
		finished = true;

		matVecMultiply( n, n, a, u, v );
		p = dotProduct( n, u, v );
		double normv = sqrt( norm2( n, v ) );

		for( i = 0; i < n; i++ )
			u[i] = v[i] / normv;
		
		if ( !nearzero( p - prev_p ) )
			finished = false;

		prev_p = p;
		iter++;
	}

	SAFEDELARR( u );
	SAFEDELARR( v );

	return p;
}

void matrixInv3D( double** mat, double** invMat )
{
	zeroMat( 3, 3, invMat );

	double detr = mat[0][0] * mat[1][1] * mat[2][2] 
					- mat[0][0] * mat[1][2] * mat[2][1]
					- mat[0][1] * mat[1][0] * mat[2][2]
					+ mat[0][1] * mat[1][2] * mat[2][0]
					+ mat[0][2] * mat[1][0] * mat[2][1]
					- mat[0][2] * mat[1][1] * mat[2][0];

	if( nearzero(detr) ) return;

	invMat[0][0] = ( mat[1][1] * mat[2][2] - mat[2][1] * mat[1][2] ) / detr;
	invMat[0][1] = ( mat[0][2] * mat[2][1] - mat[2][2] * mat[0][1] ) / detr;
	invMat[0][2] = ( mat[0][1] * mat[1][2] - mat[1][1] * mat[0][2] ) / detr;

	invMat[1][0] = ( mat[1][2] * mat[2][0] - mat[2][2] * mat[1][0] ) / detr;
	invMat[1][1] = ( mat[0][0] * mat[2][2] - mat[2][0] * mat[0][2] ) / detr;
	invMat[1][2] = ( mat[0][2] * mat[1][0] - mat[1][2] * mat[0][0] ) / detr;

	invMat[2][0] = ( mat[1][0] * mat[2][1] - mat[2][0] * mat[1][1] ) / detr;
	invMat[2][1] = ( mat[0][1] * mat[2][0] - mat[2][1] * mat[0][0] ) / detr;
	invMat[2][2] = ( mat[0][0] * mat[1][1] - mat[1][0] * mat[0][1] ) / detr;

}

void matrixInv2D( double** mat, double** invMat )
{
	zeroMat( 2, 2, invMat );

	double detr = mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
	if( nearzero(detr) ) return;

	invMat[0][0] = mat[1][1] / detr;
	invMat[0][1] = -mat[0][1] / detr;
	invMat[1][0] = -mat[1][0] / detr;
	invMat[1][1] = mat[0][0] / detr;
}

double dotProduct( int dim, double* v1, double* v2 )
{
	int i;
	double result = 0;
	for( i = 0; i < dim; i++ )
		result += v1[i] * v2[i];

	return result;
}

double dotProduct3D( double vectorA[3], double vectorB[3] )
{
	int i;
	double result = 0;
	for( i=0; i<3; i++)
		result += vectorA[i] * vectorB[i];

	return result;
}

double norm3D( double vector[3] )
{	
	return sqrt( vector[0]*vector[0] + vector[1]*vector[1] + vector[2]*vector[2] );
}

void minus3D( double vectorA[3], double vectorB[3], double result[3] )
{
	result[0] = vectorA[0] - vectorB[0];
	result[1] = vectorA[1] - vectorB[1];
	result[2] = vectorA[2] - vectorB[2];
}

void add3D( double vectorA[3], double vectorB[3], double result[3] )
{
	result[0] = vectorA[0] + vectorB[0];
	result[1] = vectorA[1] + vectorB[1];
	result[2] = vectorA[2] + vectorB[2];
}

void eye( int dim, double** mat )
{
	int i;
	for( i = 0; i < dim; i++ )
		memset( mat[i], 0, sizeof( double ) * dim );
	for( i = 0; i < dim; i++ )
		mat[i][i] = 1;
}

void zeroMat( int y, int x, double** mat )
{
	int i;
	for( i = 0; i < y; i++ )
		memset( mat[i], 0, sizeof( double ) * x );
}

bool isZeroMat( int y, int x, double** mat )
{
	bool iszero = true;
	
	int i, j;
	for( j = 0; j < y; j++ )
		for( i = 0; i < x; i++ )
			if( !nearzero( mat[j][i] ) ) iszero = false;

	return iszero;
}

void addMat( int y, int x, double** mat1, double** mat2, double** output )
{
	int i, j;
	for( j =0; j < y; j++ )
		for( i = 0; i < x; i++ )
			output[j][i] = mat1[j][i] + mat2[j][i];
}

void subMat( int y, int x, double** mat1, double** mat2, double** output )
{
	int i, j;
	for( j =0; j < y; j++ )
		for( i = 0; i < x; i++ )
			output[j][i] = mat1[j][i] - mat2[j][i];
}

void multiMat( int y, int x, int z, double** mat1, double** mat2, double** output )
{
	int i, j, k;

	for( j = 0; j < y; j++ )
		for( i = 0; i < z; i++ )
		{
			output[j][i] = 0;
			for( k = 0; k < x; k++ )
				output[j][i] += mat1[j][k] * mat2[k][i]; 
		}

}

void matVecMultiply( int y, int x, double** mat, double* vec, double* outputVec )
{
	int i, j;

	for( j = 0; j < y; j++ )
		outputVec[j] = 0;

	for( j = 0; j < y; j++ )
		for( i = 0; i < x; i++ )
			outputVec[j] += mat[j][i] * vec[i];
}

void matConstMultiply( int y, int x, double** mat, double constant, double** output )
{
	int i, j;
	
	for( j = 0; j < y; j++ )
		for( i = 0; i < x; i++ )
			output[j][i] = mat[j][i] * constant;
}

bool mat2DInverse( double** mat, double** output )
{
	int i, j;
	
	double det = mat[0][0] * mat[1][1] - mat[1][0] * mat[0][1];

	if( nearzero( det ) ) return false;

	output[0][0] =   mat[1][1] / det;
	output[0][1] = - mat[0][1] / det;
	output[1][0] = - mat[1][0] / det;
	output[1][1] =   mat[0][0] / det;

	return true;
}

void outerProduct( int dim1, double* vec1, int dim2, double* vec2, double** outMat )
{
	int i, j;

	for( j = 0; j < dim1; j++ )
		for( i = 0; i < dim2; i++ )
			outMat[j][i] = vec1[j] * vec2[i];
}

double matFDist2( int y, int x, double** mat1, double** mat2 )
{
	int i, j;
	double result = 0;
	
	for( j = 0; j < y; j++ )
		for( i = 0; i < x; i++ )
		{
			double diff = mat1[j][i] - mat2[j][i];
			result += diff * diff;
		}
	
	return result;
}

double vecDist2( int dim, double* v1, double* v2 )
{
	int i;
	double result = 0;

	for( i = 0; i < dim; i++ )
		result += (v1[i] - v2[i]) * (v1[i] - v2[i]);

	return result;
}

double norm2( int dim, double* v )
{
	int i;
	double result = 0;
	
	for( i = 0; i < dim; i++ )
		result += v[i] * v[i];

	return result;
}

void int_swap( int& a, int& b)
{
	int temp= a;
	a = b;
	b = temp;
}

char* fileExtension(char* path)
{
	if (path == NULL ) return NULL;
	char* extension = new char[1024];
	char* token = strtok( path, ". " );
	strcpy(extension, token);
	token = strtok( NULL, ". " );
	while( token != NULL )
	{
		strcpy( extension, token );
		token = strtok( NULL, ". " );
	}
	return extension;
}
